import 'package:equatable/equatable.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:osam/osam.dart';

typedef Update<T> = Function(T value);

@HiveType(typeId: 221)
class PropertyNotifier<T> extends ChangeNotifier implements ValueListenable<T> {
  PropertyNotifier(this._value);

  @HiveField(0)
  T _value;

  @override
  T get value => _value;

  set value(T newValue) {
    if (_ComparableWrapper(newValue).hashCode != _ComparableWrapper(_value).hashCode) {
      _value = newValue;
      notifyListeners();
    }
  }

  void update(Update<T> update) {
    final hashCodeBefore = _ComparableWrapper(_value).hashCode;
    update(_value);
    final hashCodeAfter = _ComparableWrapper(_value).hashCode;
    if (hashCodeAfter != hashCodeBefore) {
      notifyListeners();
    }
  }
}

class _ComparableWrapper<V> extends Equatable {
  final V _value;

  _ComparableWrapper(this._value);

  @override
  List<Object> get props => [_value];
}
